from Orange.data import Table
from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Output
import shapefile


class OWGeoLocalization(widget.OWWidget):
    name = "Geo Localization"
    description = "Provides Geolocation data (latitude/longitude)"
    icon = "icons/geolocation.svg"
    priority = 40
    shape_recs = []
    nuts_available = []
    nodes = []
    want_main_area = False
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)
    nuts_url = "https://gisco-services.ec.europa.eu/distribution/v2/nuts/" \
               "shp/NUTS_RG_20M_2021_4326.shp.zip"

    class Outputs:
        output = Output("Data", Table)

    def __init__(self):
        super().__init__()
        self.output = None

        self.combobox = gui.comboBox(
            self.controlArea, self, "", items=self.nuts_available,
            box="Select NUTS Region:",
            contentsLength=8,
        )
        self.combobox.activated.connect(self.set_coordinates)

        gui.rubber(self.controlArea)
        self.get_coordinates()
        gui.auto_commit(self.buttonsArea, self, "auto_commit", "Send",
                        box=False)

    def set_coordinates(self):
        idx_nuts = self.combobox.currentIndex()
        shp = shapefile.Reader(self.nuts_url)
        shape_rec = self.shape_recs[idx_nuts]
        polygon = shape_rec[0]

        shape = shp.shape(polygon)
        bbox = shape.bbox
        print(bbox)

        lon = bbox[0] + (bbox[2] - bbox[0])/2
        lat = bbox[1] + (bbox[3] - bbox[1])/2

        self.output = {'lon': lon,  'lat': lat}
        print(self.output)
        self.commit.deferred()

    def get_coordinates(self):
        shp = shapefile.Reader(self.nuts_url)
        self.shape_recs = [(shapeRec.record.oid,
                            f'NUTS {shapeRec.record[1]}'
                            f' - {shapeRec.record[3]}')
                           for shapeRec in
                           shp.iterShapeRecords()]
        self.nuts_available = [shapeRec[1] for shapeRec in self.shape_recs]
        self.combobox.clear()
        self.combobox.addItems(self.nuts_available)

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """
        if self.output:
            self.Outputs.output.send(self.output)

        else:
            self.Outputs.output.send(None)


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWGeoLocalization).run()



from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Output
from AnyQt.QtWidgets import QAction, QFileDialog, QPushButton
import sismic.io
from sismic.io import import_from_yaml
from sismic.model import Statechart
from Orange.data import Table
from Orange.data.pandas_compat import table_from_frame


class OWStateMachine(widget.OWWidget):
    name = "State Machine"
    description = "Reads a YAML file containing a State Machine"
    icon = "icons/statemachine.svg"
    priority = 60
    statemachine = None
    current_path = None
    want_main_area = False
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)

    class Outputs:
        output = Output("StateMachine", Statechart, auto_summary=False)

    def __init__(self):
        super().__init__()
        self.output = None

        browse_action = QAction(
            self,
            text="\N{HORIZONTAL ELLIPSIS}",
        )

        browse_action.triggered.connect(self.__show_open_dialog_file)
        browse_button = QPushButton(
            browse_action.iconText(),
            clicked=browse_action.trigger
        )

        vbox = gui.vBox(self.controlArea,
                        box="Select Simulation File:")
        hbox = gui.hBox(vbox)
        hbox.layout().addWidget(browse_button)

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self,
                        "auto_commit", "Send", box=False)

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """

        if self.current_path:
            statemachine = _load_statemachine(self.current_path)
            self.Outputs.output.send(statemachine)

        else:
            self.Outputs.output.send(None)

    def __show_open_dialog_file(self):
        self.current_path, f = QFileDialog.getOpenFileName(
            self, 'Select directory/file',
            'some/default/path/', 'Product files'
            '(*.yaml *.scxml)')
        if self.current_path:
            self.commit.deferred()


def _load_statemachine(file):

    statemachine = import_from_yaml(filepath=file)
    return statemachine


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWStateMachine).run()



from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Output, Input
from AnyQt.QtGui import QPixmap
from AnyQt.QtWidgets import QGraphicsScene, QGraphicsPixmapItem, QGraphicsView
import sismic.io
from sismic.io import import_from_yaml, export_to_plantuml
from sismic.interpreter import Interpreter
from sismic.model import Statechart
import os
import io
import sys
import plantuml
from Orange.data import Table
from Orange.data.pandas_compat import table_from_frame
import agentpy as ap
import networkx as nx
from matplotlib import pyplot as plt


class Agent(ap.Agent):

    def setup(self):
        """ Initialize a new variable at agent creation. """
        self._statemachine = self.model.statemachine
        self.states = self._statemachine.states
        self.transitions = self._statemachine.transitions
        self.events = self._statemachine.events_for()

        interpreter = Interpreter(self._statemachine)
        interpreter.execute_once()

        # State 0 is a root state
        self.actual_state = interpreter.configuration[1]
        self.event = self._statemachine.preamble

    @property
    def statemachine(self) -> Statechart:
        """
        Embedded statechart
        """
        return self._statemachine

    def update_state(self):
        for transition in self.transitions:
            if transition.source == self.actual_state and \
                    self.event == transition.event:
                self.actual_state = transition.target


class Model(ap.Model):

    def add_statemachine(self, statemachine):
        self._statemachine = statemachine

    @property
    def statemachine(self) -> Statechart:
        """
        Embedded statechart
        """
        return self._statemachine

    def setup(self):
        """ Initialize the agents and network of the model. """

        # Prepare a small-world network
        self.graph = nx.watts_strogatz_graph(
            self.p.population,
            self.p.number_of_neighbors,
            self.p.network_randomness)

        # Create agents and network
        self.agents = ap.AgentList(self, self.p.population, Agent)
        self.network = self.agents.network = ap.Network(self, self.graph)
        self.network.add_agents(self.agents, self.network.nodes)

        for i, agent in enumerate(self.agents):
            self.graph.add_node(i, agent=agent)

        # Add edges between nodes in the graph
        for i in range(self.p.population):
            for j in self.graph.neighbors(i):
                self.graph.add_edge(i, j)

        # Infect a random share of the population
        io = int(self.p.initial_infection_share * self.p.population)
        self.agents.random(io).actual_state = "infectious"
        for agent in self.agents:
            if agent.actual_state == 'infectious':
                agent.color = (0, 0, 1)
                agent.timer = 3
            if agent.actual_state == 'susceptible':
                agent.color = (0, 0.6, 1)
                agent.timer = 0

    def update(self):
        """ Record variables after setup and each step. """

        # Record share of agents with each condition
        for i, c in enumerate(('susceptible', 'infectious',
                               'recovered', 'dead')):
            n_agents = len(self.agents.select(
                self.agents.actual_state == c))
            self[c] = n_agents
            self.record(c)

        # Stop simulation if disease is gone
        if self.infectious == 0:
            self.stop()

    def step(self):
        """ Define the models' events per simulation step. """

        for agent in self.agents:
            rng = self.model.random

            for n in agent.network.neighbors(agent):
                if n.actual_state == 'infectious' and \
                        agent.actual_state == 'susceptible':
                    if self.p.infection_chance > rng.random():
                        agent.event = 'infection = True'
                        agent.timer = 3
                        agent.color = (0, 0, 1)

            if agent.actual_state == 'infectious':
                agent.timer -= 1
                if agent.timer == 0:
                    if self.p.recovery_chance > rng.random():
                        agent.event = 'recovery = True'
                        agent.color = (0.2, 1, 0)
                    else:
                        agent.event = 'recovery = False'
                        agent.color = (0, 0, 0)

        # Call 'update_state' for all agents
        self.agents.update_state()

    def end(self):
        """ Record evaluation measures at the end of the simulation. """

    def visualize(self):
        # Create a layout for the graph
        pos = nx.circular_layout(self.graph)

        # Create a new figure
        figure = plt.figure()

        # Draw the graph nodes and edges
        nx.draw_networkx_nodes(self.graph, pos,
                               node_color=[agent.color
                                           for agent in self.agents])
        nx.draw_networkx_edges(self.graph, pos)

        # Add a legend for the agent colors
        color_labels = {'susceptible': (0, 0.6, 1),
                        'infectious': (0, 0, 1), 'recovered': (0.2, 1, 0),
                        'dead': (0, 0, 0)}
        patches = [plt.plot([], [], marker="o", ms=10, ls="",
                            mec=None, color=color_labels[key],
                            label=key)[0] for key in color_labels]
        plt.legend(handles=patches, ncol=4, frameon=False)

        return figure


class OWAgent(widget.OWWidget):
    name = "Agent"
    description = "Creates an agent from State Machine"
    icon = "icons/agent.svg"
    priority = 70
    want_main_area = True
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)

    class Inputs:
        statemachine = Input("StateMachine", Statechart, auto_summary=False)

    class Outputs:
        output = Output("Model", Model, auto_summary=False)

    def __init__(self):
        super().__init__()
        self.output = None

        # Main area box
        gui.vBox(self.mainArea)
        self.main_box_scene = QGraphicsScene(self)
        self.main_box_view = QGraphicsView(self.main_box_scene)

        self.main_layout = self.mainArea.layout()
        self.main_layout.addWidget(self.main_box_view)

        self.mainArea.setMinimumWidth(100)

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self,
                        "auto_commit", "Send", box=False)

    def _create_agent_from_machine(self, statemachine):
        self.model = Model()
        self.model.add_statemachine(statemachine)
        self.agent = Agent(self.model)
        self._show_agent()

    def _show_agent(self):

        graph = sismic.io.export_to_plantuml(self.agent.statemachine)
        img = plantuml.PlantUML(
            'http://www.plantuml.com/plantuml/img/').processes(graph)

        # show a PNG thumbnail of the original image
        pixmap = QPixmap()
        pixmap.loadFromData(img)

        pic = QGraphicsPixmapItem()
        pic.setPixmap(pixmap)
        self.main_box_scene.clear()
        self.main_box_scene.addItem(pic)

    @Inputs.statemachine
    def set_statemachine(self, statemachine):
        self._create_agent_from_machine(statemachine)
        self.commit.deferred()

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """
        if self.agent:
            self.Outputs.output.send(self.model)

        else:
            self.Outputs.output.send(None)


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWAgent).run()



from Orange.widgets import widget, gui, settings
from Orange.data import Table
from Orange.widgets.widget import Output, MultiInput, Input
from Orange.data.pandas_compat import table_from_frame, table_to_frame
from pysd.py_backend.model import Model
import datetime
import pandas as pd


class OWSimulation(widget.OWWidget):
    name = "Simulation"
    description = "Executes a simulation model, allows to modify parameters " \
                  "and discrete execution"
    icon = "icons/simulation.svg"
    priority = 20
    new = 0

    stocks = set()
    flows = set()
    auxs = set()

    time_params = {
        "final_time",
        "initial_time",
        "time_step",
        "saveper",
        "time",
    }

    want_main_area = False
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)

    class Inputs:
        model = Input("Model", Model, auto_summary=False)
        data = MultiInput("Data", Table)
        location = Input("GeoLocation", Table)

    class Outputs:
        output = Output("Data", Table)

    def __init__(self):
        super().__init__()
        self.data = {}
        self.location = None
        self.output = None

        box = gui.widgetBox(self.controlArea)
        self.button = gui.button(
            box, self, "Run Simulation",
            autoDefault=False,
            callback=self.run_simulation
        )

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self,
                        "auto_commit", "Send", box=False)

    def run_simulation(self):

        if self.model:
            model = self.model

            dict_initial_value = {}
            for stock in self.stocks:

                if self.model[stock] != self.__getattribute__(stock):
                    dict_initial_value[stock] = self.__getattribute__(stock)

            dict_params = {}
            for aux in self.auxs:
                print(self.model[aux])

                if self.model[aux] != self.__getattribute__(aux):
                    dict_params[aux] = self.__getattribute__(aux)
                else:
                    dict_params[aux] = self.__getattribute__(
                        f'{aux}_duplicate')

            df = model.run(initial_condition=(self.time_start,
                                              dict_initial_value),
                           params=dict_params,
                           time_step=self.time_step,
                           final_time=self.time_stop
                           )

            indices = list(df.iloc[:, 0].keys())

            time = []
            for i in range(len(indices)):
                time.append((datetime.date(1999, 1, 1) + datetime.timedelta(
                    days=(indices[i])*7)).strftime('%Y-%m-%d %H:%M:%S'))
            df['Time'] = time

            # Add Geo Location:
            if self.location is not None:
                longitude = []
                latitude = []

                lon = self.location['lon']
                lat = self.location['lat']

                for i in range(len(indices)):
                    longitude.append(lon)
                    latitude.append(lat)

                df['Lon'] = longitude
                df['Lat'] = latitude

            self.output = table_from_frame(df)

            self.model.reload()
        self.commit.deferred()

    def _set_new_model(self, model):
        self.output = None
        self.model = model

        self.time_start = model["INITIAL TIME"]
        self.time_stop = model["FINAL TIME"]
        self.time_step = model["TIME STEP"]

        deps = self.model.dependencies
        stocks_integ = [k for k in deps.keys() if k.startswith("_integ")]
        stateful = [k for k in deps.keys() if k.startswith("_")]
        for s in stocks_integ:
            self.stocks.update(
                [
                    k.lower().replace(" ", "_")
                    for k in deps.keys()
                    if s in deps[k]
                ]
            )
            self.flows.update(
                [k.lower().replace(" ", "_") for k in deps[s]["step"].keys()]
            )
        self.auxs = {
            k.lower().replace(" ", "_")
            for k in deps.keys()
            if k not in self.stocks.union(self.flows,
                                          self.time_params,
                                          stateful)
        }

        if len(self.controlArea.children()) <= 3:

            self.time_box = gui.widgetBox(self.controlArea, "Time Controls:")
            gui.rubber(self.controlArea)
            self.stocks_box = gui.widgetBox(self.controlArea, "Stocks: ")
            gui.rubber(self.controlArea)
            self.auxs_box = gui.widgetBox(self.controlArea,
                                          "Auxiliary Values: ")

            gui.doubleSpin(self.time_box, self, "time_start",  0, 10**10,
                           decimals=4,
                           label="Start Time")
            gui.doubleSpin(self.time_box, self, "time_stop", 0, 10**10,
                           decimals=4,
                           label="Stop Time")
            gui.doubleSpin(self.time_box, self, "time_step", 0, 10**10,
                           decimals=4,
                           label="Step Time")

            for stock in self.stocks:
                self.__setattr__(stock, self.model[stock])
                gui.doubleSpin(self.stocks_box, self, stock, 0, 10**10,
                               decimals=2,
                               label=stock)

            for aux in self.auxs:
                self.__setattr__(aux, self.model[aux])
                self.__setattr__(f'{aux}_duplicate', self.model[aux])
                self.__setattr__(f'{aux}_state', False)

                gui.doubleSpin(self.auxs_box, self, aux, 0, 10**10,
                               decimals=2,
                               label=aux,)

    def set_variable(self, aux):

        df = table_to_frame(self.data[aux])
        print(df)

        lst = df.iloc[0].tolist()
        factor = len(lst) * [52]
        res = [x for x, n in zip(lst, factor) for _ in range(n)]

        var = pd.Series(index=range(len(res)), data=res)

        self.__setattr__(f'{aux}_duplicate', var)

    @Inputs.model
    def set_model(self, model):
        self._set_new_model(model)

    @Inputs.location
    def set_location(self, location):
        self.location = location

    @Inputs.data
    def set_data(self, _, table):
        self.data[table[0]] = table[1]
        self.set_variable(table[0])

    @Inputs.data.insert
    def insert_data(self, _, table):
        self.data[table[0]] = table[1]
        self.set_variable(table[0])

    @Inputs.data.remove
    def remove_data(self, index):
        if index == 0:
            self.data = {}

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """
        if self.output:
            self.Outputs.output.send(self.output)

        else:
            self.Outputs.output.send(None)


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWSimulation).run()



from AnyQt.QtWidgets import QAction, QFileDialog, QPushButton
from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Output
from pathlib import Path
from pysd.py_backend.model import Model
import pysd


class OWLoadSimulation(widget.OWWidget):
    name = "Load Simulation"
    description = "Loads a simulation workflow from the " \
                  "directory structure, supports Vensim and Xmile"
    icon = "icons/load_simulation.svg"
    priority = 10

    current_path = None
    want_main_area = False
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)

    class Outputs:
        output = Output("Model", Model, auto_summary=False)

    def __init__(self):
        super().__init__()
        self.output = None

        browse_action = QAction(
            self,
            text="\N{HORIZONTAL ELLIPSIS}",
        )

        browse_action.triggered.connect(self.__show_open_dialog_file)
        browse_button = QPushButton(
            browse_action.iconText(),
            clicked=browse_action.trigger
        )

        vbox = gui.vBox(self.controlArea, box="Select Simulation File:")
        hbox = gui.hBox(vbox)
        hbox.layout().addWidget(browse_button)

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self, "auto_commit", "Send",
                        box=False)

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """
        if self.current_path:
            model = _load_model(self.current_path)
            self.Outputs.output.send(model)

        else:
            self.Outputs.output.send(None)

    def __show_open_dialog_file(self):
        self.current_path, f = QFileDialog.getOpenFileName(
            self, 'Select directory/file', 'some/default/path/',
            'Product files(*.mdl *.xmile)')
        if self.current_path:
            self.commit.deferred()


def _load_model(file):

    path = Path(file)

    if path.suffix == ".mdl":
        model = pysd.read_vensim(file)

    elif path.suffix == ".xmile":
        model = pysd.read_xmile(file)

    else:
        raise ValueError("File format not supported")

    return model


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWLoadSimulation).run()



from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Output, Input
from Orange.data import Table
from pysd.py_backend.model import Model


class OWNameTable(widget.OWWidget):
    name = "Name Table"
    description = "Allows to rename an Orange Table"
    icon = "icons/name.svg"
    priority = 50

    stocks = set()
    flows = set()
    auxs = set()

    time_params = {
        "final_time",
        "initial_time",
        "time_step",
        "saveper",
        "time",
    }

    variables_names = []

    want_main_area = False
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)

    class Inputs:
        model = Input("Model", Model, auto_summary=False)
        data = Input("Data", Table)

    class Outputs:
        output = Output("Data", Table)

    def __init__(self):
        super().__init__()
        self.data = None
        self.model = None
        self.output = None

        self.combobox = gui.comboBox(
            self.controlArea, self, "", items=self.variables_names,
            box="Select Variable Name:",
            contentsLength=8,
        )
        self.combobox.activated.connect(self.set_table_name)

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self, "auto_commit", "Send",
                        box=False)

    def set_table_name(self):
        if self.data:
            table = self.data
            name = self.combobox.currentText()
            self.output = [name, table]
            self.commit.deferred()

    def get_variables(self, model):
        self.model = model
        self.variables_names = []

        deps = self.model.dependencies
        stocks_integ = [k for k in deps.keys() if k.startswith("_integ")]
        stateful = [k for k in deps.keys() if k.startswith("_")]
        for s in stocks_integ:
            self.stocks.update(
                [
                    k.lower().replace(" ", "_")
                    for k in deps.keys()
                    if s in deps[k]
                ]
            )
            self.flows.update(
                [k.lower().replace(" ", "_") for k in deps[s]["step"].keys()]
            )
        self.auxs = {
            k.lower().replace(" ", "_")
            for k in deps.keys()
            if k not in self.stocks.union(self.flows, self.time_params,
                                          stateful)
        }
        self.variables_names = list(self.auxs)
        self.combobox.clear()
        self.combobox.addItems([name for name in self.variables_names])

    @Inputs.model
    def set_model(self, model):
        self.get_variables(model)

    @Inputs.data
    def set_data(self, data):
        self.data = data

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """
        if self.output:
            self.Outputs.output.send(self.output)

        else:
            self.Outputs.output.send(None)


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWNameTable).run()



from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Output, Input
from AnyQt.QtGui import QPixmap, QImage
from AnyQt.QtWidgets import QGraphicsScene, QGraphicsPixmapItem, QGraphicsView
import sismic.io
from sismic.io import import_from_yaml
from sismic.interpreter import Interpreter
from Orange.data import Table
from Orange.data.pandas_compat import table_from_frame
from .agent import Model, Agent


class OWEnvironment(widget.OWWidget):
    name = "Multi Agent Environment"
    description = "Defines interactions between agents " \
                  "in a multi-agent system configuration"
    icon = "icons/environment.svg"
    priority = 80
    statemachine = None
    population = 1000
    number_of_neighbors = 2
    network_randomness = 0.5
    initial_infection_share = 0.2
    infection_chance = 0.4
    recovery_chance = 0.2
    step = 0
    time = 0
    current_path = None
    want_main_area = True
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)

    class Inputs:
        model = Input("Model", Model, auto_summary=False)

    class Outputs:
        output = Output("Data", Table)

    def __init__(self):
        super().__init__()
        self.output = None

        parameters_box = gui.widgetBox(self.controlArea, "Parameters:")

        gui.spin(parameters_box, self, "population", 0, 10 ** 8,
                 label="Number of agents:"
                 )

        gui.spin(parameters_box, self, "number_of_neighbors", 0, 100,
                 label="Number of neighbors per agent:"
                 )

        gui.doubleSpin(parameters_box, self,
                       "network_randomness", 0, 1,
                       decimals=2,
                       label="Network randomness:"
                       )

        gui.doubleSpin(parameters_box, self,
                       "initial_infection_share", 0, 1,
                       decimals=2,
                       label="Initial infection rate:"
                       )

        gui.doubleSpin(parameters_box, self,
                       "infection_chance", 0, 1,
                       decimals=2,
                       label="Infection chance:"
                       )

        gui.doubleSpin(parameters_box, self,
                       "recovery_chance", 0, 1,
                       decimals=2,
                       label="Recovery chance:"
                       )

        gui.rubber(self.controlArea)
        step_execute_box = gui.widgetBox(
            self.controlArea, "Step by step execution:")
        self.button = gui.button(
            step_execute_box, self, "Step execute",
            autoDefault=False,
            callback=self.step_execute
        )
        self.button = gui.button(
            step_execute_box, self, "Reset simulation",
            autoDefault=False,
            callback=self.reset_simulation
        )

        gui.rubber(self.controlArea)
        simulation_box = gui.widgetBox(self.controlArea)

        self.button = gui.button(
            simulation_box, self, "Run Simulation",
            autoDefault=False,
            callback=self.run_simulation
        )

        # Main area box
        gui.vBox(self.mainArea)
        self.main_box_scene = QGraphicsScene(self)
        self.main_box_view = QGraphicsView(self.main_box_scene)

        self.main_layout = self.mainArea.layout()
        self.main_layout.addWidget(self.main_box_view)

        self.mainArea.setMinimumWidth(100)

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self,
                        "auto_commit", "Send", box=False)

    def reset_simulation(self):
        self.step = 0
        self.model.sim_reset()
        self.main_box_scene.clear()
        self.output = None
        self.commit.deferred()

    def _update_sim_parameters(self):
        parameters = {'population': self.population,
                      'network_randomness':
                          self.network_randomness,
                      'number_of_neighbors':
                          self.number_of_neighbors,
                      'initial_infection_share':
                          self.initial_infection_share,
                      'recovery_chance': self.recovery_chance,
                      'infection_chance': self.infection_chance}

        return parameters

    def step_execute(self):
        if self.step == 0:
            self.model.set_parameters(self._update_sim_parameters())
            self.model.sim_setup()

        self.model.t = self.step
        self.model.sim_step()
        self.model.stop()
        self.step = self.step + 1

        self.model.create_output()
        results = self.model.output
        df = results.variables.Model

        # Visualize the graph
        self._show_graph()

        self.output = table_from_frame(df)
        self.commit.deferred()

    def run_simulation(self):

        self.model.set_parameters(self._update_sim_parameters())
        self.model.sim_reset()
        results = self.model.run(steps=100)
        self.step = self.model.t
        df = results.variables.Model

        # Visualize the graph
        self._show_graph()

        self.output = table_from_frame(df)
        self.commit.deferred()

    def _show_graph(self):

        figure = self.model.visualize()

        # Convert the Matplotlib figure to a QPixmap
        canvas = figure.canvas
        canvas.draw()
        buffer = canvas.buffer_rgba()
        qimage = QImage(buffer, canvas.get_width_height()[0],
                        canvas.get_width_height()[1],
                        QImage.Format_ARGB32)

        # Create a QGraphicsPixmapItem to display the graph
        pic = QGraphicsPixmapItem(QPixmap.fromImage(qimage))

        # Create a QGraphicsScene to hold the pixmap item
        self.main_box_scene.clear()
        self.main_box_scene.addItem(pic)

    def _set_new_environment(self, model):
        self.model = model

    @Inputs.model
    def set_model(self, model):
        self._set_new_environment(model)

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """

        if self.output:
            self.Outputs.output.send(self.output)
        else:
            self.Outputs.output.send(None)


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWEnvironment).run()



from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Output
from Orange.data import Table
from pandas.core.frame import DataFrame
from drb.drivers.eurostat import DrbEurostatFactory
from Orange.data.pandas_compat import table_from_frame


class OWEurostat(widget.OWWidget):
    name = "Eurostat"
    description = "Access and retrieve data from Eurostat service"
    icon = "icons/eurostat.svg"
    priority = 30
    variables = []
    codes = []
    want_main_area = False
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)

    class Outputs:
        output = Output("Data", Table)

    def __init__(self):
        super().__init__()
        self.service = None
        self.output = None

        box = gui.widgetBox(self.controlArea,
                            box="Enter a keyword (ex: population):")
        self.search_line = gui.lineEdit(box, self, "", controlWidth=400)

        self.button = gui.button(
            box, self, "Query Service",
            autoDefault=False,
            callback=self.query_service
        )

        gui.rubber(self.controlArea)
        self.combobox = gui.comboBox(
            self.controlArea, self, "", items=self.variables,
            box="Select Variable:",
            contentsLength=8,
        )
        self.combobox.activated.connect(self.load_variable)

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self, "auto_commit", "Send",
                        box=False)

    def query_service(self):

        factory = DrbEurostatFactory()
        self.service = factory.create('eurostat://')

        tables = self.service.get_attribute('tables')
        search = self.search_line.text()
        self.search_line.clear()

        self.variables = []
        self.codes = []

        for table in tables:
            if search in table[0].lower():

                self.variables.append(table[0])
                self.codes.append(table[1])

        self.combobox.clear()
        self.combobox.addItems([var for var in self.variables])

    def load_variable(self):
        idx = self.combobox.currentIndex()
        code = self.codes[idx]
        table = self.service[code]
        df = table.get_impl(DataFrame)

        self.output = table_from_frame(df)
        self.commit.deferred()

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """
        if self.output:
            self.Outputs.output.send(self.output)

        else:
            self.Outputs.output.send(None)


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWEurostat).run()
